-- Stillman Lectures 2, 3, AWS 2006, Tucson, March 11-15.
-------------------------------------
-- Example: rational quartic curve --
-------------------------------------
S = QQ[a..d]
P1 = QQ[s,t]
F = map(P1,S,{s^4,s^3*t,s*t^3,t^4})
I = ker F
C = res (S^1/I)
C = res I
C.dd
codim I
degree I
genus I

Ext^1(S^1/I,S)
Ext^2(S^1/I,S)
Ext^3(S^1/I,S)

------------------------------------------
-- Omega1 of the Fermat quartic surface --
------------------------------------------
kk = QQ
S = kk[a..d]
F = ideal(a^4+b^4+c^4+d^4)
R = S/F
p1 = (vars S) ** R
dj = jacobian F ** R
M = homology(p1,dj)
M = prune M -- sheaf assocuated to this is the cotangent sheaf
M = coker lift(presentation M,S)
ann M == F
res M
oo.dd
-- Therefore H^0(Omega1) = 0:
hilbertFunction({0},M) 
E2 = Ext^2(M,S^{-4})
-- h^1(Omega1) = 20
hilbertFunction({0},E2)
-- h^2(Omega1) = 0
E1 = Ext^1(M,S^{-4})
hilbertFunction({0},E1)

X = Proj R
HH^0(OO_X)
HH^1(OO_X)
HH^2(OO_X)

---------------------------------------
-- Hodge diamond of a quintic 3-fold --
---------------------------------------

omega = (I) -> (
     S := ring I;
     R := S/I;
     omegaX := prune homology(vars S ** R,jacobian gens I ** R);
     coker lift(presentation omegaX,S)
     )

cohom = (M) -> (
     -- M is an S-module
     -- Suppose that F is the corresponding sheaf
     -- Computes the list {h^0(F),h^1(F),...,h^d(F)}
     -- where d = dim of the support of M.
     d := dim M-1;
     e := euler M; -- the alternating sum of all of these
     h := for i from 1 to d list rank HH^i(sheaf M);
     h0 := e;
     s := 1;
     for i from 0 to d-1 do (h0 = h0 + s*h#i; s = -s;);
     prepend(h0, h))

hodgeDiamond3 = (I) -> (
     S := ring I;
     R := S/I;
     row1 := cohom(S^1/I);
     omegaX := omega I;
     e := euler omegaX;
     E := Ext^(numgens S-2)(omegaX, S^{-numgens S});
     h10 := row1_1;
     h13 := row1_2;
     h11 := hilbertFunction({0},E);
     h12 := e - h10 + h11 + h13;
     matrix{row1, 
	  {h10,h11,h12,h13}, 
	  {h13,h12,h11,h10},
	  reverse row1}
     )

S = QQ[a,b,c,d,e]
I = ideal(a^5+b^5+c^5+d^5+e^5)
omegaX = omega I
res omegaX
cohom (S^1/I)
hodgeDiamond3 I
omega2 = exteriorPower(2,omegaX);
res omega2
betti oo

I = ideal random(S^1, S^{-5})
hodgeDiamond3 I

I = ideal random(S^1, S^{-6})
hodgeDiamond3 I

